@@ -63,6 +63,8 @@ typedef struct {
 GMainLoop *loop = NULL;
 struct termios termios_orig;
 
+NM_CACHED_QUARK_FCN ("nmcli-error-quark", nmcli_error_quark)
+
 static void
 complete_field (GHashTable *h, const char *setting, NmcOutputField field[])
 {
@@ -89,7 +91,7 @@ complete_one (gpointer key, gpointer value, gpointer user_data)
 	else
 		last = prefix;
 
-	if ((!*last && !strchr (name, '.')) || matches (last, name) == 0) {
+	if ((!*last && !strchr (name, '.')) || matches (last, name)) {
 		g_print ("%.*s%s%s\n", (int)(last-prefix), prefix, name,
 		                       strcmp (last, name) == 0 ? "," : "");
 	}
@@ -163,18 +165,6 @@ complete_fields (const char *prefix)
 }
 
 
-/* Get an error quark for use with GError */
-GQuark
-nmcli_error_quark (void)
-{
-	static GQuark error_quark = 0;
-
-	if (G_UNLIKELY (error_quark == 0))
-		error_quark = g_quark_from_static_string ("nmcli-error-quark");
-
-	return error_quark;
-}
-
 static void
 usage (void)
 {
@@ -205,18 +195,18 @@ usage (void)
 }
 
 static const NMCCommand nmcli_cmds[] = {
-	{ "general",    do_general,     NULL },
-	{ "monitor",    do_monitor,     NULL },
-	{ "networking", do_networking,  NULL },
-	{ "radio",      do_radio,       NULL },
-	{ "connection", do_connections, NULL },
-	{ "device",     do_devices,     NULL },
-	{ "agent",      do_agent,       NULL },
-	{ NULL,         do_overview,    usage }
+	{ "general",     do_general,      NULL,   FALSE,  FALSE },
+	{ "monitor",     do_monitor,      NULL,   TRUE,   FALSE },
+	{ "networking",  do_networking,   NULL,   FALSE,  FALSE },
+	{ "radio",       do_radio,        NULL,   FALSE,  FALSE },
+	{ "connection",  do_connections,  NULL,   FALSE,  FALSE },
+	{ "device",      do_devices,      NULL,   FALSE,  FALSE },
+	{ "agent",       do_agent,        NULL,   FALSE,  FALSE },
+	{ NULL,          do_overview,     usage,  TRUE,   TRUE },
 };
 
-static NMCResultCode
-parse_command_line (NmCli *nmc, int argc, char **argv)
+static gboolean
+process_command_line (NmCli *nmc, int argc, char **argv)
 {
 	char *base;
 
@@ -253,134 +243,136 @@ parse_command_line (NmCli *nmc, int argc, char **argv)
 			}
 		}
 
-		if (matches (opt, "-terse") == 0) {
+		if (matches (opt, "-terse")) {
 			if (nmc->print_output == NMC_PRINT_TERSE) {
 				g_string_printf (nmc->return_text, _("Error: Option '--terse' is specified the second time."));
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			else if (nmc->print_output == NMC_PRINT_PRETTY) {
 				g_string_printf (nmc->return_text, _("Error: Option '--terse' is mutually exclusive with '--pretty'."));
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			else
 				nmc->print_output = NMC_PRINT_TERSE;
-		} else if (matches (opt, "-pretty") == 0) {
+		} else if (matches (opt, "-pretty")) {
 			if (nmc->print_output == NMC_PRINT_PRETTY) {
 				g_string_printf (nmc->return_text, _("Error: Option '--pretty' is specified the second time."));
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			else if (nmc->print_output == NMC_PRINT_TERSE) {
 				g_string_printf (nmc->return_text, _("Error: Option '--pretty' is mutually exclusive with '--terse'."));
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			else
 				nmc->print_output = NMC_PRINT_PRETTY;
-		} else if (matches (opt, "-mode") == 0) {
+		} else if (matches (opt, "-mode")) {
 			nmc->mode_specified = TRUE;
 			if (next_arg (&argc, &argv) != 0) {
-		 		g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
+				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "tabular", "multiline", NULL);
-			if (matches (argv[0], "tabular") == 0)
+			if (matches (argv[0], "tabular"))
 				nmc->multiline_output = FALSE;
-			else if (matches (argv[0], "multiline") == 0)
+			else if (matches (argv[0], "multiline"))
 				nmc->multiline_output = TRUE;
 			else {
-		 		g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
+				g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
-		} else if (matches (opt, "-colors") == 0) {
+		} else if (matches (opt, "-colors")) {
 			if (next_arg (&argc, &argv) != 0) {
-		 		g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
+				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "yes", "no", "auto", NULL);
-			if (matches (argv[0], "auto") == 0)
+			if (matches (argv[0], "auto"))
 				nmc->use_colors = NMC_USE_COLOR_AUTO;
-			else if (matches (argv[0], "yes") == 0)
+			else if (matches (argv[0], "yes"))
 				nmc->use_colors = NMC_USE_COLOR_YES;
-			else if (matches (argv[0], "no") == 0)
+			else if (matches (argv[0], "no"))
 				nmc->use_colors = NMC_USE_COLOR_NO;
 			else {
-		 		g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
+				g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
-		} else if (matches (opt, "-escape") == 0) {
+		} else if (matches (opt, "-escape")) {
 			if (next_arg (&argc, &argv) != 0) {
-		 		g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
+				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			if (argc == 1 && nmc->complete)
 				nmc_complete_strings (argv[0], "yes", "no", NULL);
-			if (matches (argv[0], "yes") == 0)
+			if (matches (argv[0], "yes"))
 				nmc->escape_values = TRUE;
-			else if (matches (argv[0], "no") == 0)
+			else if (matches (argv[0], "no"))
 				nmc->escape_values = FALSE;
 			else {
-		 		g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
+				g_string_printf (nmc->return_text, _("Error: '%s' is not valid argument for '%s' option."), argv[0], opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
-		} else if (matches (opt, "-fields") == 0) {
+		} else if (matches (opt, "-fields")) {
 			if (next_arg (&argc, &argv) != 0) {
-		 		g_string_printf (nmc->return_text, _("Error: fields for '%s' options are missing."), opt);
+				g_string_printf (nmc->return_text, _("Error: fields for '%s' options are missing."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			if (argc == 1 && nmc->complete)
 				complete_fields (argv[0]);
 			nmc->required_fields = g_strdup (argv[0]);
-		} else if (matches (opt, "-nocheck") == 0) {
+		} else if (matches (opt, "-nocheck")) {
 			/* ignore for backward compatibility */
-		} else if (matches (opt, "-ask") == 0) {
+		} else if (matches (opt, "-ask")) {
 			nmc->ask = TRUE;
-		} else if (matches (opt, "-show-secrets") == 0) {
+		} else if (matches (opt, "-show-secrets")) {
 			nmc->show_secrets = TRUE;
-		} else if (matches (opt, "-wait") == 0) {
+		} else if (matches (opt, "-wait")) {
 			unsigned long timeout;
 			if (next_arg (&argc, &argv) != 0) {
-		 		g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
+				g_string_printf (nmc->return_text, _("Error: missing argument for '%s' option."), opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			if (!nmc_string_to_uint (argv[0], TRUE, 0, G_MAXINT, &timeout)) {
-		 		g_string_printf (nmc->return_text, _("Error: '%s' is not a valid timeout for '%s' option."),
-				                 argv[0], opt);
+				g_string_printf (nmc->return_text, _("Error: '%s' is not a valid timeout for '%s' option."),
+						 argv[0], opt);
 				nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-				return nmc->return_value;
+				return FALSE;
 			}
 			nmc->timeout = (int) timeout;
-		} else if (matches (opt, "-version") == 0) {
+		} else if (matches (opt, "-version")) {
 			if (!nmc->complete)
 				g_print (_("nmcli tool, version %s\n"), NMCLI_VERSION);
 			return NMC_RESULT_SUCCESS;
-		} else if (matches (opt, "-help") == 0) {
+		} else if (matches (opt, "-help")) {
 			if (!nmc->complete)
 				usage ();
 			return NMC_RESULT_SUCCESS;
 		} else {
 			g_string_printf (nmc->return_text, _("Error: Option '%s' is unknown, try 'nmcli -help'."), opt);
 			nmc->return_value = NMC_RESULT_ERROR_USER_INPUT;
-			return nmc->return_value;
+			return FALSE;
 		}
 		argc--;
 		argv++;
 	}
 
 	/* Now run the requested command */
-	return nmc_do_cmd (nmc, nmcli_cmds, *argv, argc, argv);
+	nmc_do_cmd (nmc, nmcli_cmds, *argv, argc, argv);
+
+	return TRUE;
 }
 
 static gboolean nmcli_sigint = FALSE;
@@ -512,37 +504,17 @@ nmc_value_transforms_register (void)
 	                                 nmc_convert_bytes_to_string);
 }
 
-static NMClient *
-nmc_get_client (NmCli *nmc)
-{
-	GError *error = NULL;
-
-	if (!nmc->client) {
-		nmc->client = nm_client_new (NULL, &error);
-		if (!nmc->client) {
-			g_critical (_("Error: Could not create NMClient object: %s."), error->message);
-			g_clear_error (&error);
-			exit (NMC_RESULT_ERROR_UNKNOWN);
-		}
-	}
-
-	return nmc->client;
-}
-
 /* Initialize NmCli structure - set default values */
 static void
 nmc_init (NmCli *nmc)
 {
 	nmc->client = NULL;
-	nmc->get_client = &nmc_get_client;
 
 	nmc->return_value = NMC_RESULT_SUCCESS;
 	nmc->return_text = g_string_new (_("Success"));
 
 	nmc->timeout = -1;
 
-	nmc->connections = NULL;
-
 	nmc->secret_agent = NULL;
 	nmc->pwds_hash = NULL;
 	nmc->pk_listener = NULL;
@@ -589,24 +561,9 @@ nmc_cleanup (NmCli *nmc)
 	nmc_polkit_agent_fini (nmc);
 }
 
-static gboolean
-start (gpointer data)
-{
-	ArgsInfo *info = (ArgsInfo *) data;
-	info->nmc->return_value = parse_command_line (info->nmc, info->argc, info->argv);
-
-	if (!info->nmc->should_wait)
-		g_main_loop_quit (loop);
-
-	return FALSE;
-}
-
-
 int
 main (int argc, char *argv[])
 {
-	ArgsInfo args_info = { &nm_cli, argc, argv };
-
 	/* Set locale to use environment variables */
 	setlocale (LC_ALL, "");
 
@@ -628,10 +585,9 @@ main (int argc, char *argv[])
 	nmc_value_transforms_register ();
 
 	nmc_init (&nm_cli);
-	g_idle_add (start, &args_info);
-
-	loop = g_main_loop_new (NULL, FALSE);  /* create main loop */
-	g_main_loop_run (loop);                /* run main loop */
+	loop = g_main_loop_new (NULL, FALSE);
+	if (process_command_line (&nm_cli, argc, argv))
+		g_main_loop_run (loop);
 
 	if (nm_cli.complete) {
 		/* Remove error statuses from command completion runs. */
