@@ -30,6 +30,7 @@
 
 #ifndef NM_VERSION_H
 #define NM_AVAILABLE_IN_1_2
+#define NM_AVAILABLE_IN_1_8
 #endif
 
 /*
@@ -58,12 +59,14 @@
 #define NM_DBUS_INTERFACE_DHCP6_CONFIG      NM_DBUS_INTERFACE ".DHCP6Config"
 #define NM_DBUS_INTERFACE_DEVICE_INFINIBAND NM_DBUS_INTERFACE_DEVICE ".Infiniband"
 #define NM_DBUS_INTERFACE_DEVICE_BOND       NM_DBUS_INTERFACE_DEVICE ".Bond"
+#define NM_DBUS_INTERFACE_DEVICE_DUMMY      NM_DBUS_INTERFACE_DEVICE ".Dummy"
 #define NM_DBUS_INTERFACE_DEVICE_TEAM       NM_DBUS_INTERFACE_DEVICE ".Team"
 #define NM_DBUS_INTERFACE_DEVICE_VLAN       NM_DBUS_INTERFACE_DEVICE ".Vlan"
 #define NM_DBUS_INTERFACE_DEVICE_BRIDGE     NM_DBUS_INTERFACE_DEVICE ".Bridge"
 #define NM_DBUS_INTERFACE_DEVICE_GENERIC    NM_DBUS_INTERFACE_DEVICE ".Generic"
 #define NM_DBUS_INTERFACE_DEVICE_VETH       NM_DBUS_INTERFACE_DEVICE ".Veth"
 #define NM_DBUS_INTERFACE_DEVICE_TUN        NM_DBUS_INTERFACE_DEVICE ".Tun"
+#define NM_DBUS_INTERFACE_DEVICE_MACSEC     NM_DBUS_INTERFACE_DEVICE ".Macsec"
 #define NM_DBUS_INTERFACE_DEVICE_MACVLAN    NM_DBUS_INTERFACE_DEVICE ".Macvlan"
 #define NM_DBUS_INTERFACE_DEVICE_VXLAN      NM_DBUS_INTERFACE_DEVICE ".Vxlan"
 #define NM_DBUS_INTERFACE_DEVICE_GRE        NM_DBUS_INTERFACE_DEVICE ".Gre"
@@ -104,14 +107,32 @@ typedef enum {
 
 /**
  * NMState:
- * @NM_STATE_UNKNOWN: networking state is unknown
- * @NM_STATE_ASLEEP: networking is not enabled
- * @NM_STATE_DISCONNECTED: there is no active network connection
- * @NM_STATE_DISCONNECTING: network connections are being cleaned up
- * @NM_STATE_CONNECTING: a network connection is being started
- * @NM_STATE_CONNECTED_LOCAL: there is only local IPv4 and/or IPv6 connectivity
- * @NM_STATE_CONNECTED_SITE: there is only site-wide IPv4 and/or IPv6 connectivity
- * @NM_STATE_CONNECTED_GLOBAL: there is global IPv4 and/or IPv6 Internet connectivity
+ * @NM_STATE_UNKNOWN: Networking state is unknown. This indicates a daemon error
+ *    that makes it unable to reasonably assess the state. In such event the
+ *    applications are expected to assume Internet connectivity might be present
+ *    and not disable controls that require network access.
+ *    The graphical shells may hide the network accessibility indicator altogether
+ *    since no meaningful status indication can be provided.
+ * @NM_STATE_ASLEEP: Networking is not enabled, the system is being suspended or
+ *    resumed from suspend.
+ * @NM_STATE_DISCONNECTED: There is no active network connection.
+ *    The graphical shell should indicate  no network connectivity and the
+ *    applications should not attempt to access the network.
+ * @NM_STATE_DISCONNECTING: Network connections are being cleaned up.
+ *    The applications should tear down their network sessions.
+ * @NM_STATE_CONNECTING: A network connection is being started
+ *    The graphical shell should indicate the network is being connected while
+ *    the applications should still make no attempts to connect the network.
+ * @NM_STATE_CONNECTED_LOCAL: There is only local IPv4 and/or IPv6 connectivity,
+ *    but no default route to access the Internet. The graphical shell should
+ *    indicate no network connectivity.
+ * @NM_STATE_CONNECTED_SITE: There is only site-wide IPv4 and/or IPv6 connectivity.
+ *    This means a default route is available, but the Internet connectivity check
+ *    (see "Connectivity" property) did not succeed. The graphical shell should
+ *    indicate limited network connectivity.
+ * @NM_STATE_CONNECTED_GLOBAL: There is global IPv4 and/or IPv6 Internet connectivity
+ *    This means the Internet connectivity check succeeded, the graphical shell should
+ *    indicate full network connectivity.
  *
  * #NMState values indicate the current overall networking state.
  **/
@@ -123,26 +144,37 @@ typedef enum {
 	NM_STATE_CONNECTING       = 40,
 	NM_STATE_CONNECTED_LOCAL  = 50,
 	NM_STATE_CONNECTED_SITE   = 60,
-	NM_STATE_CONNECTED_GLOBAL = 70
+	NM_STATE_CONNECTED_GLOBAL = 70,
 } NMState;
 
 /**
  * NMConnectivityState:
- * @NM_CONNECTIVITY_UNKNOWN: Network connectivity is unknown.
- * @NM_CONNECTIVITY_NONE: The host is not connected to any network.
- * @NM_CONNECTIVITY_PORTAL: The host is behind a captive portal and
- *   cannot reach the full Internet.
- * @NM_CONNECTIVITY_LIMITED: The host is connected to a network, but
- *   does not appear to be able to reach the full Internet.
+ * @NM_CONNECTIVITY_UNKNOWN: Network connectivity is unknown. This means the
+ *   connectivity checks are disabled (e.g. on server installations) or has
+ *   not run yet. The graphical shell should assume the Internet connection
+ *   might be available and not present a captive portal window.
+ * @NM_CONNECTIVITY_NONE: The host is not connected to any network. There's
+ *   no active connection that contains a default route to the internet and
+ *   thus it makes no sense to even attempt a connectivity check. The graphical
+ *   shell should use this state to indicate the network connection is unavailable.
+ * @NM_CONNECTIVITY_PORTAL: The Internet connection is hijacked by a captive
+ *   portal gateway. The graphical shell may open a sandboxed web browser window
+ *   (because the captive portals typically attempt a man-in-the-middle attacks
+ *   agains the https connections) for the purpose of authenticating to a gateway
+ *   and retrigger the connectivity check with CheckConnectivity() when the
+ *   browser window is dismissed.
+ * @NM_CONNECTIVITY_LIMITED: The host is connected to a network, does not appear
+ *   to be able to reach the full Internet, but a captive portal has not been
+ *   detected.
  * @NM_CONNECTIVITY_FULL: The host is connected to a network, and
  *   appears to be able to reach the full Internet.
  */
 typedef enum {
-	NM_CONNECTIVITY_UNKNOWN,
-	NM_CONNECTIVITY_NONE,
-	NM_CONNECTIVITY_PORTAL,
-	NM_CONNECTIVITY_LIMITED,
-	NM_CONNECTIVITY_FULL
+	NM_CONNECTIVITY_UNKNOWN = 0,
+	NM_CONNECTIVITY_NONE    = 1,
+	NM_CONNECTIVITY_PORTAL  = 2,
+	NM_CONNECTIVITY_LIMITED = 3,
+	NM_CONNECTIVITY_FULL    = 4,
 } NMConnectivityState;
 
 /**
@@ -169,6 +201,8 @@ typedef enum {
  * @NM_DEVICE_TYPE_MACVLAN: a MACVLAN interface
  * @NM_DEVICE_TYPE_VXLAN: a VXLAN interface
  * @NM_DEVICE_TYPE_VETH: a VETH interface
+ * @NM_DEVICE_TYPE_MACSEC: a MACsec interface
+ * @NM_DEVICE_TYPE_DUMMY: a dummy interface
  *
  * #NMDeviceType values indicate the type of hardware represented by a
  * device object.
@@ -195,6 +229,8 @@ typedef enum {
 	NM_DEVICE_TYPE_MACVLAN    = 18,
 	NM_DEVICE_TYPE_VXLAN      = 19,
 	NM_DEVICE_TYPE_VETH       = 20,
+	NM_DEVICE_TYPE_MACSEC     = 21,
+	NM_DEVICE_TYPE_DUMMY      = 22,
 } NMDeviceType;
 
 /**
@@ -257,7 +293,7 @@ typedef enum { /*< flags >*/
  **/
 typedef enum { /*< underscore_name=nm_802_11_ap_flags, flags >*/
 	NM_802_11_AP_FLAGS_NONE    = 0x00000000,
-	NM_802_11_AP_FLAGS_PRIVACY = 0x00000001
+	NM_802_11_AP_FLAGS_PRIVACY = 0x00000001,
 } NM80211ApFlags;
 
 /**
@@ -296,7 +332,7 @@ typedef enum { /*< underscore_name=nm_802_11_ap_security_flags, flags >*/
 	NM_802_11_AP_SEC_GROUP_TKIP      = 0x00000040,
 	NM_802_11_AP_SEC_GROUP_CCMP      = 0x00000080,
 	NM_802_11_AP_SEC_KEY_MGMT_PSK    = 0x00000100,
-	NM_802_11_AP_SEC_KEY_MGMT_802_1X = 0x00000200
+	NM_802_11_AP_SEC_KEY_MGMT_802_1X = 0x00000200,
 } NM80211ApSecurityFlags;
 
 /**
@@ -316,9 +352,9 @@ typedef enum { /*< underscore_name=nm_802_11_ap_security_flags, flags >*/
  **/
 typedef enum { /*< underscore_name=nm_802_11_mode >*/
 	NM_802_11_MODE_UNKNOWN = 0,
-	NM_802_11_MODE_ADHOC,
-	NM_802_11_MODE_INFRA,
-	NM_802_11_MODE_AP
+	NM_802_11_MODE_ADHOC   = 1,
+	NM_802_11_MODE_INFRA   = 2,
+	NM_802_11_MODE_AP      = 3,
 } NM80211Mode;
 
 /**
@@ -373,7 +409,7 @@ typedef enum {
 	NM_WIMAX_NSP_NETWORK_TYPE_UNKNOWN         = 0,
 	NM_WIMAX_NSP_NETWORK_TYPE_HOME            = 1,
 	NM_WIMAX_NSP_NETWORK_TYPE_PARTNER         = 2,
-	NM_WIMAX_NSP_NETWORK_TYPE_ROAMING_PARTNER = 3
+	NM_WIMAX_NSP_NETWORK_TYPE_ROAMING_PARTNER = 3,
 } NMWimaxNspNetworkType;
 
 /**
@@ -427,7 +463,7 @@ typedef enum {
 	NM_DEVICE_STATE_SECONDARIES  = 90,
 	NM_DEVICE_STATE_ACTIVATED    = 100,
 	NM_DEVICE_STATE_DEACTIVATING = 110,
-	NM_DEVICE_STATE_FAILED       = 120
+	NM_DEVICE_STATE_FAILED       = 120,
 } NMDeviceState;
 
 /**
@@ -499,69 +535,69 @@ typedef enum {
  * Device state change reason codes
  */
 typedef enum {
-	NM_DEVICE_STATE_REASON_NONE = 0,
-	NM_DEVICE_STATE_REASON_UNKNOWN = 1,
-	NM_DEVICE_STATE_REASON_NOW_MANAGED = 2,
-	NM_DEVICE_STATE_REASON_NOW_UNMANAGED = 3,
-	NM_DEVICE_STATE_REASON_CONFIG_FAILED = 4,
-	NM_DEVICE_STATE_REASON_IP_CONFIG_UNAVAILABLE = 5,
-	NM_DEVICE_STATE_REASON_IP_CONFIG_EXPIRED = 6,
-	NM_DEVICE_STATE_REASON_NO_SECRETS = 7,
-	NM_DEVICE_STATE_REASON_SUPPLICANT_DISCONNECT = 8,
-	NM_DEVICE_STATE_REASON_SUPPLICANT_CONFIG_FAILED = 9,
-	NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED = 10,
-	NM_DEVICE_STATE_REASON_SUPPLICANT_TIMEOUT = 11,
-	NM_DEVICE_STATE_REASON_PPP_START_FAILED = 12,
-	NM_DEVICE_STATE_REASON_PPP_DISCONNECT = 13,
-	NM_DEVICE_STATE_REASON_PPP_FAILED = 14,
-	NM_DEVICE_STATE_REASON_DHCP_START_FAILED = 15,
-	NM_DEVICE_STATE_REASON_DHCP_ERROR = 16,
-	NM_DEVICE_STATE_REASON_DHCP_FAILED = 17,
-	NM_DEVICE_STATE_REASON_SHARED_START_FAILED = 18,
-	NM_DEVICE_STATE_REASON_SHARED_FAILED = 19,
-	NM_DEVICE_STATE_REASON_AUTOIP_START_FAILED = 20,
-	NM_DEVICE_STATE_REASON_AUTOIP_ERROR = 21,
-	NM_DEVICE_STATE_REASON_AUTOIP_FAILED = 22,
-	NM_DEVICE_STATE_REASON_MODEM_BUSY = 23,
-	NM_DEVICE_STATE_REASON_MODEM_NO_DIAL_TONE = 24,
-	NM_DEVICE_STATE_REASON_MODEM_NO_CARRIER = 25,
-	NM_DEVICE_STATE_REASON_MODEM_DIAL_TIMEOUT = 26,
-	NM_DEVICE_STATE_REASON_MODEM_DIAL_FAILED = 27,
-	NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED = 28,
-	NM_DEVICE_STATE_REASON_GSM_APN_FAILED = 29,
+	NM_DEVICE_STATE_REASON_NONE                           = 0,
+	NM_DEVICE_STATE_REASON_UNKNOWN                        = 1,
+	NM_DEVICE_STATE_REASON_NOW_MANAGED                    = 2,
+	NM_DEVICE_STATE_REASON_NOW_UNMANAGED                  = 3,
+	NM_DEVICE_STATE_REASON_CONFIG_FAILED                  = 4,
+	NM_DEVICE_STATE_REASON_IP_CONFIG_UNAVAILABLE          = 5,
+	NM_DEVICE_STATE_REASON_IP_CONFIG_EXPIRED              = 6,
+	NM_DEVICE_STATE_REASON_NO_SECRETS                     = 7,
+	NM_DEVICE_STATE_REASON_SUPPLICANT_DISCONNECT          = 8,
+	NM_DEVICE_STATE_REASON_SUPPLICANT_CONFIG_FAILED       = 9,
+	NM_DEVICE_STATE_REASON_SUPPLICANT_FAILED              = 10,
+	NM_DEVICE_STATE_REASON_SUPPLICANT_TIMEOUT             = 11,
+	NM_DEVICE_STATE_REASON_PPP_START_FAILED               = 12,
+	NM_DEVICE_STATE_REASON_PPP_DISCONNECT                 = 13,
+	NM_DEVICE_STATE_REASON_PPP_FAILED                     = 14,
+	NM_DEVICE_STATE_REASON_DHCP_START_FAILED              = 15,
+	NM_DEVICE_STATE_REASON_DHCP_ERROR                     = 16,
+	NM_DEVICE_STATE_REASON_DHCP_FAILED                    = 17,
+	NM_DEVICE_STATE_REASON_SHARED_START_FAILED            = 18,
+	NM_DEVICE_STATE_REASON_SHARED_FAILED                  = 19,
+	NM_DEVICE_STATE_REASON_AUTOIP_START_FAILED            = 20,
+	NM_DEVICE_STATE_REASON_AUTOIP_ERROR                   = 21,
+	NM_DEVICE_STATE_REASON_AUTOIP_FAILED                  = 22,
+	NM_DEVICE_STATE_REASON_MODEM_BUSY                     = 23,
+	NM_DEVICE_STATE_REASON_MODEM_NO_DIAL_TONE             = 24,
+	NM_DEVICE_STATE_REASON_MODEM_NO_CARRIER               = 25,
+	NM_DEVICE_STATE_REASON_MODEM_DIAL_TIMEOUT             = 26,
+	NM_DEVICE_STATE_REASON_MODEM_DIAL_FAILED              = 27,
+	NM_DEVICE_STATE_REASON_MODEM_INIT_FAILED              = 28,
+	NM_DEVICE_STATE_REASON_GSM_APN_FAILED                 = 29,
 	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_NOT_SEARCHING = 30,
-	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_DENIED = 31,
-	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_TIMEOUT = 32,
-	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_FAILED = 33,
-	NM_DEVICE_STATE_REASON_GSM_PIN_CHECK_FAILED = 34,
-	NM_DEVICE_STATE_REASON_FIRMWARE_MISSING = 35,
-	NM_DEVICE_STATE_REASON_REMOVED = 36,
-	NM_DEVICE_STATE_REASON_SLEEPING = 37,
-	NM_DEVICE_STATE_REASON_CONNECTION_REMOVED = 38,
-	NM_DEVICE_STATE_REASON_USER_REQUESTED = 39,
-	NM_DEVICE_STATE_REASON_CARRIER = 40,
-	NM_DEVICE_STATE_REASON_CONNECTION_ASSUMED = 41,
-	NM_DEVICE_STATE_REASON_SUPPLICANT_AVAILABLE = 42,
-	NM_DEVICE_STATE_REASON_MODEM_NOT_FOUND = 43,
-	NM_DEVICE_STATE_REASON_BT_FAILED = 44,
-	NM_DEVICE_STATE_REASON_GSM_SIM_NOT_INSERTED = 45,
-	NM_DEVICE_STATE_REASON_GSM_SIM_PIN_REQUIRED = 46,
-	NM_DEVICE_STATE_REASON_GSM_SIM_PUK_REQUIRED = 47,
-	NM_DEVICE_STATE_REASON_GSM_SIM_WRONG = 48,
-	NM_DEVICE_STATE_REASON_INFINIBAND_MODE = 49,
-	NM_DEVICE_STATE_REASON_DEPENDENCY_FAILED = 50,
-	NM_DEVICE_STATE_REASON_BR2684_FAILED = 51,
-	NM_DEVICE_STATE_REASON_MODEM_MANAGER_UNAVAILABLE = 52,
-	NM_DEVICE_STATE_REASON_SSID_NOT_FOUND = 53,
-	NM_DEVICE_STATE_REASON_SECONDARY_CONNECTION_FAILED = 54,
-	NM_DEVICE_STATE_REASON_DCB_FCOE_FAILED = 55,
-	NM_DEVICE_STATE_REASON_TEAMD_CONTROL_FAILED = 56,
-	NM_DEVICE_STATE_REASON_MODEM_FAILED = 57,
-	NM_DEVICE_STATE_REASON_MODEM_AVAILABLE = 58,
-	NM_DEVICE_STATE_REASON_SIM_PIN_INCORRECT = 59,
-	NM_DEVICE_STATE_REASON_NEW_ACTIVATION = 60,
-	NM_DEVICE_STATE_REASON_PARENT_CHANGED = 61,
-	NM_DEVICE_STATE_REASON_PARENT_MANAGED_CHANGED = 62,
+	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_DENIED        = 31,
+	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_TIMEOUT       = 32,
+	NM_DEVICE_STATE_REASON_GSM_REGISTRATION_FAILED        = 33,
+	NM_DEVICE_STATE_REASON_GSM_PIN_CHECK_FAILED           = 34,
+	NM_DEVICE_STATE_REASON_FIRMWARE_MISSING               = 35,
+	NM_DEVICE_STATE_REASON_REMOVED                        = 36,
+	NM_DEVICE_STATE_REASON_SLEEPING                       = 37,
+	NM_DEVICE_STATE_REASON_CONNECTION_REMOVED             = 38,
+	NM_DEVICE_STATE_REASON_USER_REQUESTED                 = 39,
+	NM_DEVICE_STATE_REASON_CARRIER                        = 40,
+	NM_DEVICE_STATE_REASON_CONNECTION_ASSUMED             = 41,
+	NM_DEVICE_STATE_REASON_SUPPLICANT_AVAILABLE           = 42,
+	NM_DEVICE_STATE_REASON_MODEM_NOT_FOUND                = 43,
+	NM_DEVICE_STATE_REASON_BT_FAILED                      = 44,
+	NM_DEVICE_STATE_REASON_GSM_SIM_NOT_INSERTED           = 45,
+	NM_DEVICE_STATE_REASON_GSM_SIM_PIN_REQUIRED           = 46,
+	NM_DEVICE_STATE_REASON_GSM_SIM_PUK_REQUIRED           = 47,
+	NM_DEVICE_STATE_REASON_GSM_SIM_WRONG                  = 48,
+	NM_DEVICE_STATE_REASON_INFINIBAND_MODE                = 49,
+	NM_DEVICE_STATE_REASON_DEPENDENCY_FAILED              = 50,
+	NM_DEVICE_STATE_REASON_BR2684_FAILED                  = 51,
+	NM_DEVICE_STATE_REASON_MODEM_MANAGER_UNAVAILABLE      = 52,
+	NM_DEVICE_STATE_REASON_SSID_NOT_FOUND                 = 53,
+	NM_DEVICE_STATE_REASON_SECONDARY_CONNECTION_FAILED    = 54,
+	NM_DEVICE_STATE_REASON_DCB_FCOE_FAILED                = 55,
+	NM_DEVICE_STATE_REASON_TEAMD_CONTROL_FAILED           = 56,
+	NM_DEVICE_STATE_REASON_MODEM_FAILED                   = 57,
+	NM_DEVICE_STATE_REASON_MODEM_AVAILABLE                = 58,
+	NM_DEVICE_STATE_REASON_SIM_PIN_INCORRECT              = 59,
+	NM_DEVICE_STATE_REASON_NEW_ACTIVATION                 = 60,
+	NM_DEVICE_STATE_REASON_PARENT_CHANGED                 = 61,
+	NM_DEVICE_STATE_REASON_PARENT_MANAGED_CHANGED         = 62,
 } NMDeviceStateReason;
 
 /**
@@ -598,13 +634,69 @@ typedef enum {
  * network.
  **/
 typedef enum {
-	NM_ACTIVE_CONNECTION_STATE_UNKNOWN = 0,
-	NM_ACTIVE_CONNECTION_STATE_ACTIVATING,
-	NM_ACTIVE_CONNECTION_STATE_ACTIVATED,
-	NM_ACTIVE_CONNECTION_STATE_DEACTIVATING,
-	NM_ACTIVE_CONNECTION_STATE_DEACTIVATED
+	NM_ACTIVE_CONNECTION_STATE_UNKNOWN      = 0,
+	NM_ACTIVE_CONNECTION_STATE_ACTIVATING   = 1,
+	NM_ACTIVE_CONNECTION_STATE_ACTIVATED    = 2,
+	NM_ACTIVE_CONNECTION_STATE_DEACTIVATING = 3,
+	NM_ACTIVE_CONNECTION_STATE_DEACTIVATED  = 4,
 } NMActiveConnectionState;
 
+/**
+ * NMActiveConnectionStateReason:
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_UNKNOWN: The reason for the active connection
+ *   state change is unknown.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_NONE: No reason was given for the active
+ *   connection state change.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_USER_DISCONNECTED: The active connection changed
+ *   state because the user disconnected it.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_DISCONNECTED: The active connection
+ *   changed state because the device it was using was disconnected.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_STOPPED: The service providing the
+ *   VPN connection was stopped.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_IP_CONFIG_INVALID: The IP config of the active
+ *   connection was invalid.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_CONNECT_TIMEOUT: The connection attempt to
+ *   the VPN service timed out.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_TIMEOUT: A timeout occurred
+ *   while starting the service providing the VPN connection.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_FAILED: Starting the service
+ *   providing the VPN connection failed.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_NO_SECRETS: Necessary secrets for the
+ *   connection were not provided.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_LOGIN_FAILED: Authentication to the
+ *   server failed.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_CONNECTION_REMOVED: The connection was
+ *   deleted from settings.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_DEPENDENCY_FAILED: Master connection of this
+ *   connection failed to activate.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_REALIZE_FAILED: Could not create the
+ *   software device link.
+ * @NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_REMOVED: The device this connection
+ *   depended on disappeared.
+ *
+ * Active connection state reasons.
+ *
+ * Since: 1.8
+ */
+NM_AVAILABLE_IN_1_8
+typedef enum {
+	NM_ACTIVE_CONNECTION_STATE_REASON_UNKNOWN                  = 0,
+	NM_ACTIVE_CONNECTION_STATE_REASON_NONE                     = 1,
+	NM_ACTIVE_CONNECTION_STATE_REASON_USER_DISCONNECTED        = 2,
+	NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_DISCONNECTED      = 3,
+	NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_STOPPED          = 4,
+	NM_ACTIVE_CONNECTION_STATE_REASON_IP_CONFIG_INVALID        = 5,
+	NM_ACTIVE_CONNECTION_STATE_REASON_CONNECT_TIMEOUT          = 6,
+	NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_TIMEOUT    = 7,
+	NM_ACTIVE_CONNECTION_STATE_REASON_SERVICE_START_FAILED     = 8,
+	NM_ACTIVE_CONNECTION_STATE_REASON_NO_SECRETS               = 9,
+	NM_ACTIVE_CONNECTION_STATE_REASON_LOGIN_FAILED             = 10,
+	NM_ACTIVE_CONNECTION_STATE_REASON_CONNECTION_REMOVED       = 11,
+	NM_ACTIVE_CONNECTION_STATE_REASON_DEPENDENCY_FAILED        = 12,
+	NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_REALIZE_FAILED    = 13,
+	NM_ACTIVE_CONNECTION_STATE_REASON_DEVICE_REMOVED           = 14,
+} NMActiveConnectionStateReason;
+
 /**
  * NMSecretAgentGetSecretsFlags:
  * @NM_SECRET_AGENT_GET_SECRETS_FLAG_NONE: no special behavior; by default no
@@ -629,14 +721,14 @@ typedef enum {
  * #NMSecretAgentGetSecretsFlags values modify the behavior of a GetSecrets request.
  */
 typedef enum { /*< flags >*/
-	NM_SECRET_AGENT_GET_SECRETS_FLAG_NONE = 0x0,
+	NM_SECRET_AGENT_GET_SECRETS_FLAG_NONE              = 0x0,
 	NM_SECRET_AGENT_GET_SECRETS_FLAG_ALLOW_INTERACTION = 0x1,
-	NM_SECRET_AGENT_GET_SECRETS_FLAG_REQUEST_NEW = 0x2,
-	NM_SECRET_AGENT_GET_SECRETS_FLAG_USER_REQUESTED = 0x4,
+	NM_SECRET_AGENT_GET_SECRETS_FLAG_REQUEST_NEW       = 0x2,
+	NM_SECRET_AGENT_GET_SECRETS_FLAG_USER_REQUESTED    = 0x4,
 
 	/* Internal to NM; not part of the D-Bus API */
-	NM_SECRET_AGENT_GET_SECRETS_FLAG_ONLY_SYSTEM = 0x80000000,
-	NM_SECRET_AGENT_GET_SECRETS_FLAG_NO_ERRORS = 0x40000000,
+	NM_SECRET_AGENT_GET_SECRETS_FLAG_ONLY_SYSTEM       = 0x80000000,
+	NM_SECRET_AGENT_GET_SECRETS_FLAG_NO_ERRORS         = 0x40000000,
 } NMSecretAgentGetSecretsFlags;
 
 /**
@@ -649,33 +741,34 @@ typedef enum { /*< flags >*/
  * #NMSecretAgentCapabilities indicate various capabilities of the agent.
  */
 typedef enum /*< flags >*/ {
-	NM_SECRET_AGENT_CAPABILITY_NONE = 0x0,
+	NM_SECRET_AGENT_CAPABILITY_NONE      = 0x0,
 	NM_SECRET_AGENT_CAPABILITY_VPN_HINTS = 0x1,
 
 	/* boundary value */
-	NM_SECRET_AGENT_CAPABILITY_LAST = NM_SECRET_AGENT_CAPABILITY_VPN_HINTS
+	NM_SECRET_AGENT_CAPABILITY_LAST      = NM_SECRET_AGENT_CAPABILITY_VPN_HINTS,
 } NMSecretAgentCapabilities;
 
 #ifndef NM_VERSION_H
 #undef NM_AVAILABLE_IN_1_2
+#undef NM_AVAILABLE_IN_1_8
 #endif
 
-#define NM_LLDP_ATTR_DESTINATION "destination"
-#define NM_LLDP_ATTR_CHASSIS_ID_TYPE "chassis-id-type"
-#define NM_LLDP_ATTR_CHASSIS_ID "chassis-id"
-#define NM_LLDP_ATTR_PORT_ID_TYPE "port-id-type"
-#define NM_LLDP_ATTR_PORT_ID "port-id"
-#define NM_LLDP_ATTR_PORT_DESCRIPTION "port-description"
-#define NM_LLDP_ATTR_SYSTEM_NAME "system-name"
-#define NM_LLDP_ATTR_SYSTEM_DESCRIPTION "system-description"
-#define NM_LLDP_ATTR_SYSTEM_CAPABILITIES "system-capabilities"
-#define NM_LLDP_ATTR_IEEE_802_1_PVID "ieee-802-1-pvid"
-#define NM_LLDP_ATTR_IEEE_802_1_PPVID "ieee-802-1-ppvid"
-#define NM_LLDP_ATTR_IEEE_802_1_PPVID_FLAGS "ieee-802-1-ppvid-flags"
-#define NM_LLDP_ATTR_IEEE_802_1_VID "ieee-802-1-vid"
-#define NM_LLDP_ATTR_IEEE_802_1_VLAN_NAME "ieee-802-1-vlan-name"
+#define NM_LLDP_ATTR_DESTINATION             "destination"
+#define NM_LLDP_ATTR_CHASSIS_ID_TYPE         "chassis-id-type"
+#define NM_LLDP_ATTR_CHASSIS_ID              "chassis-id"
+#define NM_LLDP_ATTR_PORT_ID_TYPE            "port-id-type"
+#define NM_LLDP_ATTR_PORT_ID                 "port-id"
+#define NM_LLDP_ATTR_PORT_DESCRIPTION        "port-description"
+#define NM_LLDP_ATTR_SYSTEM_NAME             "system-name"
+#define NM_LLDP_ATTR_SYSTEM_DESCRIPTION      "system-description"
+#define NM_LLDP_ATTR_SYSTEM_CAPABILITIES     "system-capabilities"
+#define NM_LLDP_ATTR_IEEE_802_1_PVID         "ieee-802-1-pvid"
+#define NM_LLDP_ATTR_IEEE_802_1_PPVID        "ieee-802-1-ppvid"
+#define NM_LLDP_ATTR_IEEE_802_1_PPVID_FLAGS  "ieee-802-1-ppvid-flags"
+#define NM_LLDP_ATTR_IEEE_802_1_VID          "ieee-802-1-vid"
+#define NM_LLDP_ATTR_IEEE_802_1_VLAN_NAME    "ieee-802-1-vlan-name"
 
-#define NM_LLDP_DEST_NEAREST_BRIDGE "nearest-bridge"
+#define NM_LLDP_DEST_NEAREST_BRIDGE          "nearest-bridge"
 #define NM_LLDP_DEST_NEAREST_NON_TPMR_BRIDGE "nearest-non-tpmr-bridge"
 #define NM_LLDP_DEST_NEAREST_CUSTOMER_BRIDGE "nearest-customer-bridge"
 
