@@ -38,17 +38,18 @@
 #include "nm-dbus-interface.h"
 #include "NetworkManagerUtils.h"
 #include "nm-manager.h"
-#include "nm-linux-platform.h"
+#include "platform/nm-linux-platform.h"
 #include "nm-bus-manager.h"
-#include "nm-device.h"
-#include "nm-dhcp-manager.h"
+#include "devices/nm-device.h"
+#include "dhcp/nm-dhcp-manager.h"
 #include "nm-config.h"
 #include "nm-session-monitor.h"
 #include "nm-dispatcher.h"
-#include "nm-settings.h"
+#include "settings/nm-settings.h"
 #include "nm-auth-manager.h"
 #include "nm-core-internal.h"
 #include "nm-exported-object.h"
+#include "dns/nm-dns-manager.h"
 #include "systemd/nm-sd.h"
 
 #if !defined(NM_DIST_VERSION)
@@ -58,6 +59,8 @@
 #define NM_DEFAULT_PID_FILE          NMRUNDIR "/NetworkManager.pid"
 #define NM_DEFAULT_SYSTEM_STATE_FILE NMSTATEDIR "/NetworkManager.state"
 
+#define CONFIG_ATOMIC_SECTION_PREFIXES ((char **) NULL)
+
 static GMainLoop *main_loop = NULL;
 static gboolean configure_and_quit = FALSE;
 
@@ -85,8 +88,9 @@ _set_g_fatal_warnings (void)
 }
 
 static void
-_init_nm_debug (const char *debug)
+_init_nm_debug (NMConfig *config)
 {
+	gs_free char *debug = NULL;
 	const guint D_RLIMIT_CORE = 1;
 	const guint D_FATAL_WARNINGS = 2;
 	GDebugKey keys[] = {
@@ -96,6 +100,11 @@ _init_nm_debug (const char *debug)
 	guint flags;
 	const char *env = getenv ("NM_DEBUG");
 
+	debug = nm_config_data_get_value (nm_config_get_data_orig (config),
+	                                  NM_CONFIG_KEYFILE_GROUP_MAIN,
+	                                  NM_CONFIG_KEYFILE_KEY_MAIN_DEBUG,
+	                                  NM_MANAGER_RELOAD_FLAGS_NONE);
+
 	flags  = nm_utils_parse_debug_string (env, keys, G_N_ELEMENTS (keys));
 	flags |= nm_utils_parse_debug_string (debug, keys, G_N_ELEMENTS (keys));
 
@@ -160,7 +169,7 @@ print_config (NMConfigCmdLineOptions *config_cli)
 
 	nm_logging_setup ("OFF", "ALL", NULL, NULL);
 
-	config = nm_config_new (config_cli, NULL, &error);
+	config = nm_config_new (config_cli, CONFIG_ATOMIC_SECTION_PREFIXES, &error);
 	if (config == NULL) {
 		fprintf (stderr, _("Failed to read configuration: %s\n"), error->message);
 		return 7;
@@ -272,7 +281,7 @@ main (int argc, char *argv[])
 
 		/* don't free these strings, we need them for the entire
 		 * process lifetime */
-		nm_dhcp_helper_path = g_strdup_printf ("%s/src/dhcp-manager/nm-dhcp-helper", path);
+		nm_dhcp_helper_path = g_strdup_printf ("%s/src/dhcp/nm-dhcp-helper", path);
 
 		g_free (path);
 	}
@@ -293,7 +302,7 @@ main (int argc, char *argv[])
 	}
 
 	/* Read the config file and CLI overrides */
-	config = nm_config_setup (config_cli, NULL, &error);
+	config = nm_config_setup (config_cli, CONFIG_ATOMIC_SECTION_PREFIXES, &error);
 	nm_config_cmd_line_options_free (config_cli);
 	config_cli = NULL;
 	if (config == NULL) {
@@ -302,7 +311,7 @@ main (int argc, char *argv[])
 		exit (1);
 	}
 
-	_init_nm_debug (nm_config_get_debug (config));
+	_init_nm_debug (config);
 
 	/* Initialize logging from config file *only* if not explicitly
 	 * specified by commandline.
@@ -362,7 +371,10 @@ main (int argc, char *argv[])
 #endif
 	             );
 
-	nm_auth_manager_setup (nm_config_get_auth_polkit (config));
+	nm_auth_manager_setup (nm_config_data_get_value_boolean (nm_config_get_data_orig (config),
+	                                                         NM_CONFIG_KEYFILE_GROUP_MAIN,
+	                                                         NM_CONFIG_KEYFILE_KEY_MAIN_AUTH_POLKIT,
+	                                                         NM_CONFIG_DEFAULT_MAIN_AUTH_POLKIT_BOOL));
 
 	nm_manager_setup ();
 
@@ -390,6 +402,8 @@ main (int argc, char *argv[])
 		goto done;
 	}
 
+	nm_platform_process_events (NM_PLATFORM_GET);
+
 	/* Make sure the loopback interface is up. If interface is down, we bring
 	 * it up and kernel will assign it link-local IPv4 and IPv6 addresses. If
 	 * it was already up, we assume is in clean state.
@@ -425,6 +439,8 @@ done:
 
 	nm_config_state_set (config, TRUE, TRUE);
 
+	nm_dns_manager_stop (nm_dns_manager_get ());
+
 	if (global_opt.pidfile && wrote_pidfile)
 		unlink (global_opt.pidfile);
 
