@@ -30,7 +30,7 @@
 
 #include "nm-auth-subject.h"
 #include "nm-config.h"
-#include "nm-settings-connection.h"
+#include "settings/nm-settings-connection.h"
 
 /*****************************************************************************/
 
@@ -302,11 +302,11 @@ _nm_audit_manager_log_generic_op (NMAuditManager *self, const char *file, guint
 void
 _nm_audit_manager_log_device_op (NMAuditManager *self, const char *file, guint line,
                                  const char *func, const char *op, NMDevice *device,
-                                 gboolean result, gpointer subject_context,
+                                 gboolean result, const char *args, gpointer subject_context,
                                  const char *reason)
 {
 	gs_unref_ptrarray GPtrArray *fields = NULL;
-	AuditField interface_field = { }, ifindex_field = { };
+	AuditField interface_field = { }, ifindex_field = { }, args_field = { };
 	int ifindex;
 
 	g_return_if_fail (op);
@@ -324,6 +324,11 @@ _nm_audit_manager_log_device_op (NMAuditManager *self, const char *file, guint l
 		g_ptr_array_add (fields, &ifindex_field);
 	}
 
+	if (args) {
+		_audit_field_init_string (&args_field, "args", args, FALSE, BACKEND_ALL);
+		g_ptr_array_add (fields, &args_field);
+	}
+
 	_audit_log_helper (self, fields, file, line, func, op, result, subject_context, reason);
 }
 
