@@ -223,8 +223,6 @@ bss_proxy_get_properties (NMSupplicantInterface *self, GDBusProxy *proxy)
 	return g_variant_builder_end (&builder);
 }
 
-#define BSS_PROXY_INITED "bss-proxy-inited"
-
 static void
 bss_proxy_acquired_cb (GDBusProxy *proxy, GAsyncResult *result, gpointer user_data)
 {
@@ -1126,7 +1124,7 @@ nm_supplicant_interface_disconnect (NMSupplicantInterface * self)
 
 	/* Cancel all pending calls related to a prior connection attempt */
 	if (priv->assoc_data) {
-		gs_free_error GError *error = NULL;
+		gs_free GError *error = NULL;
 
 		nm_utils_error_set_cancelled (&error, FALSE, "NMSupplicantInterface");
 		assoc_return (self, error, "abort due to disconnect");
@@ -1603,7 +1601,7 @@ dispose (GObject *object)
 	NMSupplicantInterfacePrivate *priv = NM_SUPPLICANT_INTERFACE_GET_PRIVATE (self);
 
 	if (priv->assoc_data) {
-		gs_free_error GError *error = NULL;
+		gs_free GError *error = NULL;
 
 		nm_utils_error_set_cancelled (&error, TRUE, "NMSupplicantInterface");
 		assoc_return (self, error, "cancelled due to dispose of supplicant interface");
